char *usage = "$Id: mpbutil.c,v 1.6 2006/12/01 09:49:41 axel Exp $\n"
"mpbutil does not modify anything - it just reads stdin and writes to stdout and stderr"
" and prints commands to be executed by the admin who knows what he's doing"
"1. read the MPB - this command gives commands to read the MPB\n"
"./mpbutil -g\n"
"2. edit the MPB - this mods the MPB and give commands to rebuild/patch\n"
"./mpbutil </etc/sda.mpb >/etc/sdaclean.mpb\n";

#define NUMDISKS 2
#define NUMRAIDS 2

#include <stdio.h>
#include <assert.h>
#include </usr/src/linux/include/linux/hdreg.h>
#include "mpbutil.h"

/* Things we can depend on ICHxR Intel RAID OROM not to violate */
#define MAX_RAID_ARRAYS 8
#define MAX_RAID_VOLUMES 16
#define MAX_RAID_MEMBER_DISKS 8

/* ataraid.c uses bits in variable ataraiduse to keep track of raid devices */
#define MAX_ATARAID_RAIDDEVS (sizeof(unsigned int) * 8)

#define DEGRADED_MAP 2
#define FAILED_MAP   3

#define SPARE_DISK      0x01  /* Spare */
#define CONFIGURED_DISK 0x02  /* Member of some RaidDev */
#define FAILED_DISK     0x04  /* Permanent failure */
#define USABLE_DISK     0x08  /* Fully usable unless FAILED_DISK is set */

#define DISABLED_BIT 0
#define DEGRADED_BIT 1
#define FAILED_BIT   2   /* should match FAILED_DISK's bit, above */

#define DISK_LINE_LEN 128    /* overkill */

static u32 compute_checksum(const u32 *buffer, u32 mpb_size)
{
	u32 i = 0, sum = -((struct _raid_mpb *) buffer)->check_sum;
	for (mpb_size /= sizeof(u32); i < mpb_size; i++)
		sum += *buffer++;
	return sum;
}

int main(int argc, char *argv[]) {
struct raid_mpb mpb;
u32 cksum;
int i, j;
char sdisk = '.';
char tdisk = '.';
u32 start = 0;
u32 blocks = 0;
u32 total = 0;

if (argc==2 && !strcmp(argv[1], "-h")) {
 fprintf(stderr, usage);
 exit(2);
 }

if (argc==2 && !strcmp(argv[1], "-g")) {
 system("perl -n -e 'if (/^\\s+(\\d+)\\s+(\\d+)\\s+(\\d+)\\s+(sd[a-d])\\s+/) {\n"
	" $major = $1;\n"
	" $minor = $2;\n"
	" $blocks = $3;\n"
	" $sdev = $4;\n"
	" $mpb = $blocks * 2 - 2;\n"
	" printf \"dd if=/dev/%s of=/etc/%s1.mpb skip=%d bs=1b count=1\\n\", $sdev, $sdev, $mpb;\n"
	" printf \"dd if=/dev/%s of=/etc/%s2.mpb skip=%d bs=1b count=1\\n\", $sdev, $sdev, $mpb-1;\n"
	" printf \"cat /etc/%s1.mpb /etc/%s2.mpb >/etc/%s.mpb\\n\", $sdev, $sdev, $sdev;\n"
	" }' </proc/partitions");
 exit(2);
 }

fread(&mpb, 512, 2, stdin);
fprintf(stderr, "Intel Signature        : %-*.*s\n", MAX_SIGNATURE_LENGTH, MAX_SIGNATURE_LENGTH, &mpb.sig);
fprintf(stderr, "Size of MPB            : %12lu\n", mpb.mpb_size);
fprintf(stderr, "MPB Checksum           : %12lu\n", mpb.check_sum);
cksum = compute_checksum((u32 *) &mpb, mpb.mpb_size);
fprintf(stderr, "Checksum computed      : %12lu\n", cksum);
assert(mpb.check_sum == cksum);
fprintf(stderr, "family_num             : %12lu Cksum from 1st time this config was written\n", mpb.family_num);
fprintf(stderr, "generation_num         : %12lu Incremented each time array's MPB is written\n", mpb.generation_num);
assert(mpb.num_disks == NUMDISKS);
fprintf(stderr, "num_disks              : %12d\n", mpb.num_disks);
assert(mpb.num_raid_devs == NUMRAIDS);
fprintf(stderr, "num_raid_devs          : %12d\n", mpb.num_raid_devs);

for (i=0; i<NUMDISKS; i++) {
 fprintf(stderr, "\nDisk %d serial number   : %-*.*s\n", i, MAX_RAID_SERIAL_LEN, MAX_RAID_SERIAL_LEN, &mpb.disk_tbl[i].serial);
 fprintf(stderr, "total_blocks           : %12lu\n", mpb.disk_tbl[i].total_blocks);
 fprintf(stderr, "scsi_id                : %08x\n", mpb.disk_tbl[i].scsi_id);
 fprintf(stderr, "status                 : %08x", mpb.disk_tbl[i].status);
 if (mpb.disk_tbl[i].status != 0x13a) {
  fprintf(stderr, " <=== 0x13a");
  mpb.disk_tbl[i].status = 0x13a;
  tdisk = 'a' + (mpb.disk_tbl[i].scsi_id >> 16);
  }
 else {
  sdisk = 'a' + (mpb.disk_tbl[i].scsi_id >> 16);
  total = mpb.disk_tbl[i].total_blocks;
  }
 fprintf(stderr, "\n");
 }

for (i=0; i<NUMRAIDS; i++) {
 fprintf(stderr, "\nRaid %d serial number   : %-*.*s\n", i, MAX_RAID_SERIAL_LEN, MAX_RAID_SERIAL_LEN, &mpb.raid_tbl[i].serial);
 fprintf(stderr, "num_data_blocks_lo     : %12lu\n", mpb.raid_tbl[i].num_data_blocks_lo);
 fprintf(stderr, "num_data_blocks_hi     : %12lu\n", mpb.raid_tbl[i].num_data_blocks_hi);
 fprintf(stderr, "status                 : %08x", mpb.raid_tbl[i].status);
 if (mpb.raid_tbl[i].status != mpb.raid_tbl[i].raid_vol.lo_map.raid_level ? 0x00 : 0x0c) {
  fprintf(stderr, " <=== %08x", mpb.raid_tbl[i].raid_vol.lo_map.raid_level ? 0x00 : 0x0c);
  mpb.raid_tbl[i].status = mpb.raid_tbl[i].raid_vol.lo_map.raid_level ? 0x00 : 0x0c;
  }
 fprintf(stderr, "\n");
 fprintf(stderr, "reserved_blocks        : %12lu\n", mpb.raid_tbl[i].reserved_blocks);
 fprintf(stderr, "migr_state             : %12d", mpb.raid_tbl[i].raid_vol.migr_state);
 if (mpb.raid_tbl[i].raid_vol.migr_state != 0) {
  fprintf(stderr, " <=== 0");
  mpb.raid_tbl[i].raid_vol.migr_state = 0;
  }
 fprintf(stderr, "\n");
 fprintf(stderr, "migr_type              : %12d", mpb.raid_tbl[i].raid_vol.migr_type);
 if (mpb.raid_tbl[i].raid_vol.migr_type != mpb.raid_tbl[i].raid_vol.lo_map.raid_level) {
  fprintf(stderr, " <=== %12d", mpb.raid_tbl[i].raid_vol.lo_map.raid_level);
  mpb.raid_tbl[i].raid_vol.migr_type = mpb.raid_tbl[i].raid_vol.lo_map.raid_level;
  }
 fprintf(stderr, "\n");
 fprintf(stderr, "dirty                  : %12d\n", mpb.raid_tbl[i].raid_vol.dirty);
 fprintf(stderr, "pba_of_lba0            : %12lu\n", mpb.raid_tbl[i].raid_vol.lo_map.pba_of_lba0);
 fprintf(stderr, "blocks_per_member      : %12lu\n", mpb.raid_tbl[i].raid_vol.lo_map.blocks_per_member);
 fprintf(stderr, "num_data_stripes       : %12lu\n", mpb.raid_tbl[i].raid_vol.lo_map.num_data_stripes);
 fprintf(stderr, "blocks_per_strip       : %12lu\n", mpb.raid_tbl[i].raid_vol.lo_map.blocks_per_strip);
 fprintf(stderr, "map_state              : %04x", mpb.raid_tbl[i].raid_vol.lo_map.map_state);
 if (mpb.raid_tbl[i].raid_vol.lo_map.map_state != 0) {
  fprintf(stderr, " <=== 0");
  mpb.raid_tbl[i].raid_vol.lo_map.map_state = 0;
  if (mpb.raid_tbl[i].raid_vol.lo_map.raid_level == 1) {
   blocks = mpb.raid_tbl[i].raid_vol.lo_map.blocks_per_member;
   start = mpb.raid_tbl[i].raid_vol.lo_map.pba_of_lba0;
   }
  }
 fprintf(stderr, "\n");
 fprintf(stderr, "raid_level             : %1d\n", mpb.raid_tbl[i].raid_vol.lo_map.raid_level);
 fprintf(stderr, "num_members            : %1d\n", mpb.raid_tbl[i].raid_vol.lo_map.num_members);
 for (j=0; j<NUMDISKS; j++) {
  fprintf(stderr, "disk_ord_tbl[%1d]        : %08x", j, mpb.raid_tbl[i].raid_vol.lo_map.disk_ord_tbl[j]);
  if ((mpb.raid_tbl[i].raid_vol.lo_map.disk_ord_tbl[j] & 0xffffff00) != 0) {
   fprintf(stderr, " <=== %08x", mpb.raid_tbl[i].raid_vol.lo_map.disk_ord_tbl[j] & 0xff);
   mpb.raid_tbl[i].raid_vol.lo_map.disk_ord_tbl[j] = mpb.raid_tbl[i].raid_vol.lo_map.disk_ord_tbl[j] & 0xff;
   }
  fprintf(stderr, "\n");
  }
 }
cksum = compute_checksum((u32 *) &mpb, mpb.mpb_size);
if (mpb.check_sum != cksum) {
 mpb.generation_num++;
 cksum = compute_checksum((u32 *) &mpb, mpb.mpb_size);
 fprintf(stderr, "MPB Checksum           : %12lu <=== %12lu\n\n", mpb.check_sum, cksum);
 fprintf(stderr, "dd if=/dev/sd%c of=/dev/sd%c bs=1b count=%lu skip=%lu\n", sdisk, tdisk, blocks, start);
 fprintf(stderr, "dd if=/etc/sd%cclean.mpb of=/dev/sd%c bs=1b count=1 skip=1 seek=%lu\n", sdisk, sdisk, total-3);
 fprintf(stderr, "dd if=/etc/sd%cclean.mpb of=/dev/sd%c bs=1b count=1 skip=0 seek=%lu\n", sdisk, sdisk, total-2);
 fprintf(stderr, "dd if=/etc/sd%cclean.mpb of=/dev/sd%c bs=1b count=1 skip=1 seek=%lu\n", sdisk, tdisk, total-3);
 fprintf(stderr, "dd if=/etc/sd%cclean.mpb of=/dev/sd%c bs=1b count=1 skip=0 seek=%lu\n", sdisk, tdisk, total-2);
 fprintf(stderr, "dd if=/dev/sd%c of=/etc/sd%c1n.mpb bs=1b count=1 skip=0 seek=%lu\n", sdisk, sdisk, total-2);
 fprintf(stderr, "dd if=/dev/sd%c of=/etc/sd%c2n.mpb bs=1b count=1 skip=0 seek=%lu\n", sdisk, sdisk, total-3);
 fprintf(stderr, "cat /etc/sd%c[12]n.mpb >/etc/sd%cn.mpb\n", sdisk, sdisk);
 fprintf(stderr, "dd if=/dev/sd%c of=/etc/sd%c1n.mpb bs=1b count=1 skip=0 seek=%lu\n", tdisk, tdisk, total-2);
 fprintf(stderr, "dd if=/dev/sd%c of=/etc/sd%c2n.mpb bs=1b count=1 skip=0 seek=%lu\n", tdisk, tdisk, total-3);
 fprintf(stderr, "cat /etc/sd%c[12]n.mpb >/etc/sd%cn.mpb\n", tdisk, tdisk);
 fprintf(stderr, "cmp -l /etc/sd%c.mpb /etc/sd%cn.mpb\n", sdisk, sdisk);
 fprintf(stderr, "cmp -l /etc/sd%c.mpb /etc/sd%cn.mpb\n", tdisk, tdisk);
 fprintf(stderr, "cmp -l /etc/sd%c.mpb /etc/sd%c.mpb\n", sdisk, tdisk);
 fprintf(stderr, "cmp -l /etc/sd%cn.mpb /etc/sd%cn.mpb\n", sdisk, tdisk);
 mpb.check_sum = cksum;
 fwrite(&mpb, 512, 2, stdout);
 exit(1);
 }
exit(0);
}
