typedef unsigned int u32;
typedef unsigned short u16;
typedef unsigned char u8;

#include "iswraid.h"

/* RAID MAP CONFIGURATION INFO */
struct mpb_raid_map {
	u32 pba_of_lba0;	// start address of partition
	u32 blocks_per_member;	// blocks per member
	u32 num_data_stripes;	// number of data stripes
	u16 blocks_per_strip;
	u8 map_state;		// Normal, Uninitialized, Degraded, Failed
	u8 raid_level;		// 0, 1
	u8 num_members;		// number of member disks
	u8 reserved[3];
	u32 filler[MPB_RAID_MAP_FILLERS];	// expansion area
	u32 disk_ord_tbl[NUMDISKS];	// disk_ord_tbl[num_members], top byte special
};

/* RAID VOLUME INFO */
struct mpb_raid_vol {
	u32 reserved[2];
	u8 migr_state;		// Normal or Migrating
	u8 migr_type;		// Initializing, Rebuilding, ...
	u8 dirty;
	u8 fill[1];
#define	MPB_RAID_VOL_FILLERS	(5)	// expansion area keeps the loMap
	u32 filler[MPB_RAID_VOL_FILLERS];
	struct mpb_raid_map lo_map;
	// here comes another one if migr_state
};

/* RAID DEVICE CONFIGURATION INFO */
struct mpb_raid_dev {
	struct mpb_serial serial;	// serial number
	u32 num_data_blocks_lo;	// Data blocks on device (low 32 bits)
	u32 num_data_blocks_hi;	// Data blocks on device (high 32 bits)
	u32 status;		// Persistent RaidDev status
	u32 reserved_blocks;	// Reserved blocks at beginning of volume
#define	MPB_RAID_DEV_FILLERS	(12)
	u32 filler[MPB_RAID_DEV_FILLERS];
	struct mpb_raid_vol raid_vol;
};

struct raid_mpb {
	struct raid_cfg_sig sig;
	u32 check_sum;		// MPB Checksum
	u32 mpb_size;		// Size of MPB
	u32 family_num;		// Cksum from 1st time this config was written
	u32 generation_num;	// Incremented each time array's MPB is written
	u32 reserved[2];
	u8 num_disks;		// Number of configured disks
	u8 num_raid_devs;	// Number of configured volumes
	u8 fill[2];
	u32 filler[RAID_MPB_FILLERS];
	struct _mpb_disk disk_tbl[NUMDISKS];	// disk_tbl[num_disks]
	struct mpb_raid_dev raid_tbl[NUMRAIDS];
	u8 fillerb[512];
};
